/*
 * This is the data logging program for the HWS bat house project using the HWS bathouse shield.
 * This Adruino shield supports the following features:
 * NE555 oscillator circuit for IR signal generation
 * Four IR photo interrupter bat entry/exit detectors
 * Four DS 18S20 1-wire temperature sensors
 * One Microchip 24AA1025 128K serial EEPROM data storage
 * SD card slot for removable data storage
 * On-board DS 1307 RTC with battery backup 
 * CdS light sensor
 * Other digital and analog sensors as desired.
 *
 * To keep the user interface very simple, we use a menu driven command line interface using the built-in
 * Arduino serial window.  This allows choosing sensors to log, logging device options, simple debugging feedback,
 * RTC setup, and other basic utilties.
 *
 * Design and implementation of the Bathouse Shield is derived from open source contributions by the good folks at AdaFruit, fatlib, and 
 * RTClib library, released under the respective GNU and Creative Commons licenses.
 * http://www.adafruit.com
 * http://code.google.com/p/sdfatlib/
 * https://github.com/adafruit/RTClib
 * Thanks to everyone who contributed to those projects.
 * This code is released under the Attribution-ShareAlike license from Creative Commons. 
 *
 * Programmers/Designers: Josh Davis, Tomas Carvalho y Silvia, and John Vaughn.
 * Hobart and William Smith Colleges, Geneva, NY 14456
 * Last modified: 1/4/11
 */

#include <avr/pgmspace.h>
#include <EEPROM.h>
#include <Wire.h>
#include <OneWire.h>
#include <SdFat.h>

// Useful odds and ends
#define NOT_FOUND -1
#define NULL_CHAR '\0'
#define SPACE     ' '
#define CR        '\r'

// Should we wait for a carriage return from our serial command terminal?
// If so, change to 1 and recompile.
#define WAIT_FOR_CR 0

// Set our serial baud
#define BAUD   19200

// SD card error codes
#define NO_ERROR 0
#define CARD_ERROR -1
#define FILE_ERROR -2
#define WRITE_ERROR -3
#define SYNC_ERROR -4

#define STRING_MAX 32

// Modes here for determining if user interacts via main menu or just begin logging
#define START    0
#define MENU     1
#define LOGGING  2

// Seconds to wait for initial user choice
#define COUNTDOWN 10

// Defines for setting various timers
#define MILLISECONDS 1
#define HUNDRETHSECONDS 10
#define TENTHSECONDS 100
#define SECONDS 1000
#define MINUTES 60000L
#define HOURS 3600000L

// Constants to indicate asynchronous, synchronous, and reset events that caused logging to happen.
#define RESET_EVENT  0
#define SYNCH_EVENT  1
#define ASYNCH_EVENT 2

// Typical limit on different types of sensors we can log.  Notice that four thermometers count as 
// a single sensor type, single RTC is split into date and time, etc.
#define SENSOR_TYPES 8
#define IR_SENSORS   4

// LED display I/O pin for indicating we are alive
#define LED_PIN      4
#define LED_ON       digitalWrite(LED_PIN,HIGH)
#define LED_OFF      digitalWrite(LED_PIN,LOW)

// Pin numbers for IR detector sensor lines.  Note, digital pins 10, 11 are reserved for SD card!
#define IR_0_PIN     5
#define IR_1_PIN     6
#define IR_2_PIN     8
#define IR_3_PIN     9

// Pin for RTC using 
#define RTC_PIN      7

// Pin number for analog light level
#define LIGHT_PIN    0

// Global variables here for handling user input, timing, etc
int mode;                                   // User interaction modes: start, menu, or logging
long startTime;                             // Elapsed time mark
char myChar;                                // Serial input chars
int currIndex;                               // Current index to add next char to strBuffer
int irIndex;
char strBuffer[STRING_MAX];                  // Utility array used for user input, atoi conversion, etc

// Global variables here are restored from Arduino EEPROM on startup which allows unattended logging on restart.
boolean logThisSensor[SENSOR_TYPES]; // Flags for determining which sensors 0-7 to log
int IRState[IR_SENSORS] = {
  LOW,LOW,LOW,LOW};         // State flags for IR events
unsigned long IRTimer[IR_SENSORS] = {
  0L,0L,0L,0L};   // Records of an IR event duration
int tempAvail;                 // Holds the number of available 1-Wire thermometers
byte tempAddr[4][8];           // Temperature sensor chip addresses 
float prevTemp[4];             // Holds previous temp values for quick access in asynch log events
boolean logSEEPROM, logSD;     // Flags to indicate which storage devices used for data logging
unsigned int logInterval;      // Time to wait between synchronous log
char sdFileName[13];           // SD card FAT file name (8.3) to use are prototype for log file
unsigned long seepromCurrent;  // Next place in SEEPROM to write, avoiding earlier data
unsigned long itemNum;         // Initial entry number in order of being logged. Allows arbitrary numbering starts.
boolean previewSensors;        // Flag to show logged data as written, used for debugging mostly

// Reset activity timer to zero.  We need to reset our general purpose msec timer to wait for synchronous events.
void resetTimer() {
  startTimer(millis());
}  // End resetTimer

// Set our timer reference from the msec parameter.  Parameter is usually millis() for system clock timer.
void startTimer(long msec) {
  startTime = msec;
}  // End startTimer

// Returns the millisecond count elapsed thus far in the desired units.  Parameter choices are:
// MILLISECONDS, HUNDRETHSECONDS, TENTHSECONDS, SECONDS, MINUTES, and HOURS.  
// So elapsedTime(TENTHSECONDS) returns
// 100 when 10 seconds has elapsed from last resetTimer().
long elapsedTime(long units) {
  return (millis() - startTime) / units;
}  // End elapsedTime

// Utility routine.  Erase all prior input to our buffer and null terminate it.  We return length 0 so
// we can reset our string length at the same time.
int flushBuffer(char a[]) {
  for (int i=0; i<STRING_MAX; i++)
    a[i]=' ';
  a[0] = NULL_CHAR;
  return 0;
}  // End of flushBuffer

// Simple filter for serial input chars.  We convert all to lowercase and ignore spaces and most low ASCII chars.
// For chars we ignore, return the null char.  We return carriage return as is to
// faciliate support to serial programs sending chars as typed and only one CR at the end.
char filter(char c) {
  if (c == CR) return c;
  else if (c <= SPACE) return NULL_CHAR;
  return tolower(c);
}  // End filter

// Collection of initialization of variables here.  Called by setup on reboot/startup so 
// we restore our values stored in 
// Arduino EEPROM and set some system configuration items.
void initialize() {
  pinMode(LED_PIN,OUTPUT);     // Display we are alive and logging
  pinMode(IR_0_PIN,INPUT);     // Set up our infrared detector pin inputs
  pinMode(IR_1_PIN,INPUT);
  pinMode(IR_2_PIN,INPUT);
  pinMode(IR_3_PIN,INPUT);
  digitalWrite(IR_0_PIN,HIGH);  // Activate the pull-up resistors on the inputs
  digitalWrite(IR_1_PIN,HIGH);
  digitalWrite(IR_2_PIN,HIGH);
  digitalWrite(IR_3_PIN,HIGH);
  IRState[0] = digitalRead(IR_0_PIN);  // Get intial reads so we can start in known states.
  IRState[1] = digitalRead(IR_1_PIN);
  IRState[2] = digitalRead(IR_2_PIN);
  IRState[3] = digitalRead(IR_3_PIN);
  getCurrentSettings("no");    // Reset variables from our EEPROM but don't show it now.  
  mode = START;                // Begin loop() in start mode so we delay for menu selection
  if (initSD(sdFileName) == NO_ERROR) Serial.println("SD card OK");   // Check status of SD card
  else Serial.println("SD card error"); 
  LED_ON;  
}  // End initialize

// Setup simply gets our Wiring, Serial, inits variables and starts a timer to wait for user input.
void setup() {
  Wire.begin();
  Serial.begin(BAUD);      // Open the serial port:
  initialize();            // Do our variable initializations
  Serial.println("Enter any char for main menu.");
  startTimer(millis());    // Ready to start logging or menu
}  // End setup

/* Main loop begins here.  On startup we display a prompt and wait a few seconds for user input. If we receive none, go directly into
 * logging mode and log data using the previous EEPROM restored settings.  If we get any user input, show the main menu and
 * go into menu mode. We never return to start mode but either are in menu or logging mode from now on.  We can switch between these
 * two modes by user input.  Note, data is logged in two situations.  Regular (synchronous) data is logged on a periodic interval
 * set by the user.  Asynchronous data logging happens when IR break beam events happen.  
 */

void loop() {
  // Capture any user input and react accordingly
  while (Serial.available() > 0) {// Add any incoming Serial characters to our buffer
    myChar = filter(Serial.read());  // Filter our chars to ignore spaces, convert to lower case, recognize CR if needed
    if (myChar > CR) strBuffer[currIndex++] = myChar;    // Good input, add to buffer
    currIndex %= STRING_MAX;                             // Wrap around for large input, no overruns
    strBuffer[currIndex] = NULL_CHAR;  // Manually add string terminator as temporary end of input
    delay(10);  // Give some time so we get an entire user input string if no CR is coming
  } // End while still have input to process
  // If we need to wait for a CR to terminate our input, do so by looping again
  #if WAIT_FOR_CR
     if (myChar != CR) return;
  #endif
     
  // Check which mode we are in and react accordingly
  switch (mode) {
  case START:   // Start mode normally occurs only once after reset.  Just wait for any user input until timeout.
    if ( (strlen(strBuffer) > 0) && (elapsedTime(SECONDS) < COUNTDOWN)) {  // Got input?
      mode = MENU;       // Get ready for menu mode.
      showMainMenu();
      currIndex = flushBuffer(strBuffer);
    }  // End if
    else if ( elapsedTime(SECONDS) > COUNTDOWN ) {  // No user input, we time out and go directly to logging
      mode = LOGGING;
    }  // End if
    break;  // End of start mode

  case MENU:                 // Handles input meant to indicate menu choices with parameters attached
    if ( elapsedTime(TENTHSECONDS)%10 < 5 ) LED_ON; 
    else LED_OFF;  // Blink to show we are in menu mode
    if (strlen(strBuffer) > 0) {                    // Have a non-empty string to process
      int command = getCommand(strBuffer[0]);       // Get the initial command character, match latter with our command list
      strBuffer[0] = ' ';                           // Erase our first char so we can now do an atol on the rest
      for (int i = 0; i < strlen(strBuffer)-1; i++)  { // Collect remaining user input into our command parameter string            
        strBuffer[i] = strBuffer[i+1];            
      }  // End for copying chars to strBuffer as parameter
      strBuffer[currIndex-1] = NULL_CHAR;           // Set new terminator
      if (doCommand(command,strBuffer))           // Carry out the command indicated, may be ill formed, catch in this function call
        Serial.println("Operation successful.");  // Indicate success/failure to user
      else
        Serial.println("Error in operation."); 
      currIndex = flushBuffer(strBuffer);          // Done with input, clear buffer to avoid atol hangovers
    }  // End if we read some serial bytes
    break;  // End of menu mode

  case LOGGING:                             // Start logging data here 
    if (strlen(strBuffer) > 0) {            // First, check to make sure the user is not typing some chars to interrupt logging.
      mode = MENU;
      showMainMenu();
      currIndex = flushBuffer(strBuffer);  // If the user enters anything, return to menu mode
      break;
    }  // End if
    // Now actual logging events begin
    irIndex = updateIR();          // Any IR events recorded?
    if ( irIndex != NOT_FOUND ) {  // Check asynchronous events status and log any such event
      logData(ASYNCH_EVENT);  
      clearIR(irIndex);            // Logged this event, clear it to start anew
    }  // 
    if ( elapsedTime(SECONDS)%10 == 1 ) LED_ON; 
    else LED_OFF;                   // Blink once to show we are logging
    if (elapsedTime(TENTHSECONDS) > logInterval) {    // Log when our time interval is reached
      logData(SYNCH_EVENT);        // Do the logging now
      resetTimer();                // Restart our ms timer so we know when to log again
    }  // End if 
    break;
  default:                     // Unknown mode.  Should not get here without some weird error
    ;
  }  // End switch
}  // End loop
// End of Bathouse file
