/*
File Name: Config
 Contents: Code for saving settings in Ardiuno EEPROM
 Last modified: 1/5/11
 Programmer: John Vaughn
 Comments: This code declares some progmem strings to prompt users for changes to Arduino EEPROM changes and manages the 
 activity of making and restoring those EEPROM settings.
 */



// These defines set Arduino EEPROM addresses where we save non-volatile configuration information.
#define SENSOR_0          0
#define SENSOR_1          1
#define SENSOR_2          2
#define SENSOR_3          3
#define SENSOR_4          4
#define SENSOR_5          5
#define SENSOR_6          6
#define SENSOR_7          7
#define RESERVED          8
#define THERMS            9
#define THERM_0           10
#define THERM_1           18
#define THERM_2           26
#define THERM_3           34
#define RESERVED          42
#define RESERVED          43
#define LOGGING_SEEPROM   50
#define LOGGING_SD        51
#define LOGGING_INTERVAL  52
#define LOG_FILE_NAME     54
#define RESERVED          66
#define SEEPROM_CURRENT   67
#define LINE_NUMBER       71
#define PREVIEW_SENSORS   75

// Define the size of the following progmem labels for showing Arduion EERPROM values.
#define SETTING_TABLE_SIZE 13

// Declaring progmem strings for our settings display.
prog_char settingLabel0[] PROGMEM = "Sensors: ";
prog_char settingLabel1[] PROGMEM = "Therms: ";
prog_char settingLabel2[] PROGMEM = "Therm 0 @: ";
prog_char settingLabel3[] PROGMEM = "Therm 1 @: ";
prog_char settingLabel4[] PROGMEM = "Therm 2 @: ";
prog_char settingLabel5[] PROGMEM = "Therm 3 @: ";
prog_char settingLabel6[] PROGMEM = "Log SEEPROM: ";
prog_char settingLabel7[] PROGMEM = "Logging SD: ";
prog_char settingLabel8[] PROGMEM = "Log interval (0.1 secs): ";
prog_char settingLabel9[] PROGMEM = "Log file name: ";
prog_char settingLabel10[] PROGMEM = "SEEPROM data: ";
prog_char settingLabel11[] PROGMEM = "Next rec num: ";
prog_char settingLabel12[] PROGMEM = "Preview: ";

// Add the strings to a string table for our menu choices text
PROGMEM const char *settingTable[] = 	  
{   
  settingLabel0,
  settingLabel1,
  settingLabel2,
  settingLabel3,
  settingLabel4,
  settingLabel5,
  settingLabel6,
  settingLabel7,
  settingLabel8,
  settingLabel9,
  settingLabel10,
  settingLabel11,
  settingLabel12
};  // End of setting string table declaration

/* 
Routines begin here for setting our Arduino EEPROM values which are used on reset to restore addresses of devices,
number of devices to log, etc.  We use EEPROM.h heavily here to do our EEPROM reads and writes.  We write MSB of
parameters to lowest EEPROM addresses.  For example, to store 0x1234 into two EEPROM bytes starting at addess 98, we
write 0x12 to address 98 and write 0x34 to address 99.
*/

// Select which sensors to log as desired. Entry of any negative parameter means exclude all.  Params 0-7 adds that sensor.
// We return true to indicate noError, false means some error occurred.
boolean selectSensor(int sensorNum) {  // Select/deselect a sensor to include in logs
  int i;
  if (sensorNum < SENSOR_0) {
    for (i=SENSOR_0; i<=SENSOR_7;i++) {
      EEPROM.write(i, false);
    }  // Negative parameter zeroes everything
    return true;                                // Done without error
  }  // End if
  if (sensorNum >= SENSOR_TYPES) return false;  // Out of range for a device number, error condition
  EEPROM.write(sensorNum, true);                // Write to EEPROM positive information    
  return true;
}  // End selectSensor

// Determine which storage devices for logging data. String in param can begin: "see" (SEEPROM), "sd" (SD card), or "none" (no logging).
// This mainly uses the strstr C-string library function to locate a substring in our parameter string.
boolean setLogDevice(char param[]) {
  if ( strstr(param,"see") > 0) {            // Does str start with substring for seeprom?
    EEPROM.write(LOGGING_SEEPROM, 0xFF);     // We log to Serial EEPROM
    return true;
  }  // End if
  if ( strstr(param,"sd") > 0) {              // Similar for SD
    EEPROM.write(LOGGING_SD, 0xFF);          // We log to SD card
    return true;
  }  // End if
  if ( strstr(param,"no") > 0 ) {          // None chosen
    EEPROM.write(LOGGING_SEEPROM, 0x00);   // We don't log to any storage
    EEPROM.write(LOGGING_SD, 0x00);
    return true;
  }  // End if
  return false;            // Fall through error to indicate unknown parameter
}  // End setLogDevice

// Record the logging interval we want between standard, synchronous, log events.  Parameter interval units 
// are in 1/10 seconds.
boolean setLogInterval(unsigned int interval) {
  if (interval > 0) {
    EEPROM.write(LOGGING_INTERVAL, highByte(interval));  
    EEPROM.write(LOGGING_INTERVAL+1, lowByte(interval));
    return true;
  }  // If valid interval
  return false;
}  // End setLogInterval

// Set the real time clock.  Format of allowed param is sec/min/hour/dayOfWeek/dayOfMonth/month/year, where each item is 
// a two digit integer. param string must be 20 chars, 14 digits + 6 separators.  Typical param: 00/05/21/01/28/09/10
boolean setRTC(char param[]) {
  char buffer[8] = "       ";  // Char array for atoi
  byte timeDate[7];           // Byte array for sec,minute,hour,dayOfWeek,dayOfMonth,month,year
  int timeIndex = 0;          // Cursor for adding items to timeDate array
  
  if ( strlen(param) < 20) {  // Check for errors
    Serial.println("Too short");
    return false;
  }
  for (int i = 0; i < 20; i++) {    // Work through the numbers in mod 3 order, converting and checking for separators
    switch (i%3) {
    case 0: 
      buffer[0] = param[i];   // Add tens digit to buffer
      break;
    case 1:
      buffer[1] = param[i];   // Add units digit to buffer 
      timeDate[timeIndex++] = atoi(buffer); // Convert to byte and put into timeDate
      break;
    case 2: 
      if ( param[i] !='/') return false;
    } // End switch
  }  // End for
  // Write to the chip to set the current time and date
  setDateDs1307(timeDate[0],timeDate[1],timeDate[2],timeDate[3],timeDate[4],timeDate[5],timeDate[6]);
    return true;
  }  // End End of setRTC

// Set the current filename.ext for logging to an SD card.  Filename must be standard FAT 8+3 twelve char format exactly, such as logger00.csv.
// No spaces, 12 chars with period exactly right or we fail. NOTE: We overwrite the last two file name chars of the name on reset 
// so we start creating file names such as logger01.csv, logger02.csv, etc.  This prevents complete logging failure if we 
// have an open/full/corrupted SD file.
boolean setLogFilename( char param[] ) {
  if ( strlen(param) != 12 ) return false;   // Check exact length
  for (int i = 0; i < 12; i++) {
    if ( param[i] == ' ') return false;       // Check for no spaces anywhere in name
  }  // End for checking chars
  if ( param[8] != '.' ) return false;        // Check the separator
  for (int i = 0; i < 12; i++) {               // Okay, accept the name as our template
    EEPROM.write(LOG_FILE_NAME + i, param[i]);
  }  // End if
  return true;
}  // End setLogFilename

// Set next address where we will log to in SEEPROM.  Allows us to return to prior logging address on restart.
boolean setSEEPROM(unsigned long param) {
    word part = param >> 16;
    EEPROM.write(SEEPROM_CURRENT, highByte(part));   // MSBs of address go in first
    EEPROM.write(SEEPROM_CURRENT+1, lowByte(part));
    part = (int) param;
    EEPROM.write(SEEPROM_CURRENT+2, highByte(part)); // LSBs go in at higher addresses
    EEPROM.write(SEEPROM_CURRENT+3, lowByte(part));
    return true;
}  // End setSEEPROM

// Set the initial data item number to be used when resuming logging.  Here we can begin our recording at any desired 
// record number.  This means we do not lose track of our record numbering on restart.
boolean setLogRecordStart(unsigned long count) {
  word part = count >> 16;
  EEPROM.write(LINE_NUMBER, highByte(part));  // Similar to other EEPROM writes
  EEPROM.write(LINE_NUMBER+1, lowByte(part));
  part = (int) count;
  EEPROM.write(LINE_NUMBER+2, highByte(part));
  EEPROM.write(LINE_NUMBER+3, lowByte(part));
  return true;
}  // End of setLogRecordStart

// Flag to display sensor data only to serial window, not to logging devices.  Mostly for 
// debugging sensors prior to deployment.  Parameter containing "y" designates yes, all others mean no.
boolean setSensorPreview(char param[]) {
  byte b = ( strstr(param,"y") > 0 ? 0xFF : 0x00 );
  EEPROM.write(PREVIEW_SENSORS, b);   // Mark for display of sensors only);
    return true;
}  // End setSensorPreview

// Show the current Arduino EEPROM settings for our various device settings.  Allows us to both write EEPROM values and display them
// as well if desired.  Parameter containing substring "no" means do not show.  All others default to meaning yes, show them.
// Getter functions both set the values and do the display as well.
boolean getCurrentSettings(char param[]) {
  char buffer[40];    // Holds menu item string read from progmem
  boolean show = ( strstr(param,"no" ) > 0 ? false : true );    // Explicitly avoid showing them?
  if (show) {
    Serial.println("Current settings...");  // Display notice of intent
    drawSeparator(80);                      // Draw a separator line of given length
  }  // End if show
  for (int item = 0; item < SETTING_TABLE_SIZE; item++)  // Run through our settings label string table
  {
    strcpy_P(buffer, (char*)pgm_read_word(&(settingTable[item]))); // Copy string table entry into buffer.
    if (show) Serial.print(buffer);        // First print the label then the appropriate values
    
      switch (item) {    // Now do the writes and display the results as well if so chosen by parameter show.
      case 0:  
        getSelectedSensors(show);
        break;
      case 1:  
        getThermCount(show);
        break; 
      case 2:  
        getThermAddr(show, 0);
        break;
      case 3:
        getThermAddr(show,1);
        break;  
      case 4:  
        getThermAddr(show, 2);
        break;
      case 5:  
        getThermAddr(show, 3);
        break;
      case 6:  
        getLogSEEPROM(show);
        break;
      case 7: 
        getLogSD(show);
        break;
      case 8:  
        getLogInterval(show);
        break;
      case 9:  
        getLogFileName(show);
        break;
      case 10:  
        getSEEPROMCurrent(show);
        break;
      case 11:  
        getNextItemNumber(show);
        break;
        case 12: 
        getPreviewSensors(show);
        break;
      default : ;
      }  // End case
  }  // End loop through labels
  return true;  // Indicate no error
}  // End getCurrentSettings

// Getter functions begin here which read EEPROM values and restore our global variables so we can quickly
// reference them without going to EEPROM for laborious reads to determine settings.  We display these variables if requested by parameter show.

// Read/show selected sensors by their index.  Output of 0235 means we have selected only sensors 0, 2, 3, and 5 to log. 
void getSelectedSensors(boolean show) {
  for(int i= SENSOR_0; i <= SENSOR_7; i++) {if (EEPROM.read(i) != 0) {
  logThisSensor[i] = true;} else logThisSensor[i] = false; 
  if (show && logThisSensor[i]) Serial.print(i);
  }
  if (show) Serial.println();
}  // End getSelectedSensors

// Utility function for checking to see if we should be logging a specific sensor
boolean sensorSelected(int n) {
  return logThisSensor[n];
}  // End sensorSelected

// Read/show number of thermometers we have located when we did our initial settings.
void getThermCount(boolean show) {
  tempAvail = (int) EEPROM.read(THERMS);
  if (show) Serial.println(tempAvail);
}  // End getThermCount

// Read/show therm 8 byte addresses for a specific thermometer of index n.
void getThermAddr(boolean show, int n) {
  int addr = 0;
  switch (n) {  // Only four therm addresses possible.  Read EEPROM to get them.
    case 0: addr = THERM_0;break;
    case 1: addr = THERM_1;break;
    case 2: addr = THERM_2;break;
    case 3: addr = THERM_3;break;
  }  // End switch
  for (int i = 0; i < 8; i++) {tempAddr[n][i] = EEPROM.read(addr + i); if (show) Serial.print(tempAddr[n][i],HEX);}  // Show the addresses of therms
  if (show) Serial.println();
}  // End getThermAddr

// Read/Show true/false if logging to SEEPROM
void getLogSEEPROM(boolean show) {
  logSEEPROM = ( EEPROM.read(LOGGING_SEEPROM) == 0xFF ? true : false);
  if (show) if (logSEEPROM) Serial.println("true"); else Serial.println("false");
}  // End getLogSEEPROM

// Read/Show if logging to SD
void getLogSD(boolean show) {
  logSD = ( EEPROM.read(LOGGING_SD) == 0xFF ? true : false);
  if (show) if (logSD) Serial.println("true"); else Serial.println("false");
}  // End of getLogSD

// Show interval between loggings
void getLogInterval(boolean show) {
  logInterval = EEPROM.read(LOGGING_INTERVAL);
  logInterval = (logInterval << 8);
  logInterval += EEPROM.read(LOGGING_INTERVAL+1);
  if (show) Serial.println(logInterval);
}  // End of getLogInterval

// Show file name we are logging to as recorded in EEPROM
void getLogFileName(boolean show) {
  for (int i = 0; i < 12; i++) {sdFileName[i] = EEPROM.read(LOG_FILE_NAME + i);} sdFileName[12] = '\0'; if (show) Serial.print(sdFileName);
  if (show) Serial.println();
}  // End of getLogFileName

// Show current SEEPROM address where we next will write when logging to SEEPROM
void getSEEPROMCurrent(boolean show){
    unsigned long i = 0L;
      i |= EEPROM.read(SEEPROM_CURRENT);
      i = i << 8;
      i |= EEPROM.read(SEEPROM_CURRENT+1);
      i = i << 8;
      i |= EEPROM.read(SEEPROM_CURRENT+2);
      i = i << 8;
      i |= EEPROM.read(SEEPROM_CURRENT+3);
      seepromCurrent = i;
      if (show) Serial.println(seepromCurrent);
}  // End of getSEEPROM

// Show next record item number we will use when creating a record to log
void  getNextItemNumber(boolean show){
  unsigned long i = 0L;
      i |= EEPROM.read(LINE_NUMBER);
      i = i << 8;
      i |= EEPROM.read(LINE_NUMBER+1);
      i = i << 8;
      i |= EEPROM.read(LINE_NUMBER+2);
      i = i << 8;
      i |= EEPROM.read(LINE_NUMBER+3);
      itemNum = i;
      if (show) Serial.println(itemNum);
}  // End of getNextItemNumber

// Show true/false for data preview as we log.  Mainly for debugging.
void getPreviewSensors(boolean show) {
      previewSensors = (EEPROM.read(PREVIEW_SENSORS) ==0xFF ? true : false);
      if (show) if (previewSensors) Serial.println("true"); else Serial.println("false");
}  // End of getPreviewSensors

// End of config file
























































