/*
 Contents: Code for displaying menu choices and handling user menu interactions.
 Last modified: 1/5/11
 Programmer: John Vaughn
 Comments: This section of code is the menu display and very trivial command interpreter.  We declare our progmem string table for
 menu option listings and have functions to carry out the corresponding commands from the menu. We have a clunky labeling/command
 scheme for commands as one char codes.  These are members of a string we use as a basic lookup, returning the number code of the command
 as the string index.  Once a command is recognized, we call a function to carry it out, which is where parsing the command parameter
 is done as well.
*/

// Flags to indicate where we direct EEPROM output
#define SERIAL 0
#define SD     1

// Number of menu items
#define MENU_SIZE 14

// Command defines here match the index of the menu labels and also the index of menu strings in progmem string table
#define SENSOR   0
#define THERM    1
#define RTC      2
#define STORAGE  3
#define FILENAME 4
#define INTERVAL 5
#define RECORD   6
#define SHOW     7
#define COPY     8
#define ERASE    9
#define PREVIEW  10
#define SETTINGS 11
#define MENU     12
#define LOG      13

// Formatting defines
#define TWO_TABS                           "\t\t"
#define THREE_TABS                         "\t\t\t"

// Declaring progmem strings for our menu display.  The index of these strings should match the 
// index of the menu choices defined below: 'a' <=> index 0, 'b' <=> index 1, etc.
prog_char menuItem0[] PROGMEM = "Select sensors.";
prog_char menuItem1[] PROGMEM = "Init thermometers.";
prog_char menuItem2[] PROGMEM = "Set real time clock.";
prog_char menuItem3[] PROGMEM = "Choose storage.";
prog_char menuItem4[] PROGMEM = "Set SD filename.";
prog_char menuItem5[] PROGMEM = "Set log interval.";
prog_char menuItem6[] PROGMEM = "Set record number.";
prog_char menuItem7[] PROGMEM = "Show SEEPROM data.";
prog_char menuItem8[] PROGMEM = "Copy SEEPROM to SD.";
prog_char menuItem9[] PROGMEM = "Set SEEPROM addr.";
prog_char menuItem10[] PROGMEM = "Preview data.";
prog_char menuItem11[] PROGMEM = "Show settings.";
prog_char menuItem12[] PROGMEM = "Refresh menu.";
prog_char menuItem13[] PROGMEM = "Start logging.";

// Add the strings to a string table for our menu choices text
PROGMEM const char *menuTable[] = 	  
{   
  menuItem0,
  menuItem1,
  menuItem2,
  menuItem3,
  menuItem4,
  menuItem5,
  menuItem6,
  menuItem7,
  menuItem8,
  menuItem9,
  menuItem10,
  menuItem11,
  menuItem12,
  menuItem13
};  // End of menu string table declaration

// Utility functions go here to manage menu choices, formatting, command lookups, etc.

// Use a loop to draw separator lines for menu.  We do this to avoid strings being created in SRAM.
void drawSeparator(int length) {
  for (int i=0; i < length; i++) Serial.print("-"); 
  Serial.println();
}  // End of drawSeparator

// Menu choice chars.  Each character's index in this string MUST match the menu text string indices defined above.
// For example, 'a' here at string index 0 must match desired menuTable[0] string content.
char menuChars[] = "abcdefimnopqrs";

// Main menu printing begins here. Notice we use tabs to arrange our menu into two columns.  We use a display order array
// to handle the string table entries in mod 2 fashion.  
// This gives us a two column format based on the order in our display array. 
void showMainMenu() {
  byte menuDisplayOrder[MENU_SIZE] = {
    SENSOR, SHOW, THERM, COPY, RTC, ERASE, STORAGE, PREVIEW, FILENAME, SETTINGS, INTERVAL, MENU, RECORD, LOG        };
  char menuChoice;    // Menu commands 
  char buffer[STRING_MAX];  // Holds menu item string read from progmem

  drawSeparator(80);  // Draw a divider line for clarity

  for (int i = 0; i < MENU_SIZE; i++) {  // Walk through menuDisplay in order and use the various indices to display a menu choice and text.
    menuChoice =  (menuChars[menuDisplayOrder[i]]);   // Get the menu char choice.
    strcpy_P(buffer, (char*)pgm_read_word(&(menuTable[menuDisplayOrder[i]]))); // Copy string table entry into buffer, from Arduino progmem reference.
    Serial.print(menuChoice);  // Show the menu choice character
    Serial.print(". ");        // Show separator symbols
    Serial.print(buffer);      // Show the menu choice text itself
    Serial.print(THREE_TABS);  // Spacing between columns
    if (i%2 == 1) Serial.println();  // Form a two column menu to save vertical space
  }
  drawSeparator(80);  // Draw another divider line for clarity
  Serial.println("Type letter command and any required parameter.");  // Issue the terse prompt
}  // End showMainMenu

// Run through our menu choice character string and look for a match to our user input first char ch.  We return the 
// index position in the string as our command integer code.  Lazy programmers lookup table.
int getCommand(char ch) {
  for (int i = 0; i < MENU_SIZE; i++) if (menuChars[i] == ch) return i;  // Found the character the user input began with
  return NOT_FOUND;  // Unknown command character entered
}  // End getCommand

// Determine user command and parameter(s) from serial stream input and try to carry out the indicated command.
// We assume valid input for most commands, but atol conversions may fail and gave zero paramInt value. 
// Fine, we just indicate failure if we need 
// non-zero parameter.  Some params are non-numeric strings so we pass the original parameter string along in that case.
// Each function returns boolean value of true for noError and false for some kind of failure.

boolean doCommand(int command, char paramStr[STRING_MAX]){
  boolean noError = true;  // Assume the best
  long paramInt = atol(strBuffer);             // Convert any parameter found to long, may be unnecessary, we don't check here but pass it along as needed
  switch (command) {
  case SENSOR: // Set the devices to include in logs
    noError = selectSensor(paramInt);
    break;
  case THERM: // Set the number and addresses of the available thermometers
    noError = (getThermAddresses() > 0);
    break;
  case  RTC: 
    noError = setRTC(paramStr);    // Set the real time clock setting
    break;
  case  STORAGE:  
    noError = setLogDevice(paramStr);  // Set which devices we log to
    break;
  case  INTERVAL: 
    noError = setLogInterval(paramInt);  // Set interval between logs
    break;
  case  SETTINGS: 
    getCurrentSettings(paramStr);      // Get/show current EEPROM settings
    break;
  case SHOW:                           // Display indicated number of bytes of EEPROM contents
    noError = showSEEPROM(seepromCurrent,paramInt,SERIAL);   
    break;
  case COPY:                          // Copy EEPROM contents to current SD card file
    if (writeSDStr("From SEEPROM") == NO_ERROR) noError = showSEEPROM(seepromCurrent,paramInt,SD);
    else noError = false;    // SD card trouble
    break;
  case  ERASE: 
    noError = setSEEPROM(paramInt);  // Set EERPROM write at indicated address
    break;
  case  PREVIEW: 
    noError = setSensorPreview(paramStr);    // Show data as logged
    break;
  case  FILENAME:                    // Set filename for SD card  
    noError = setLogFilename(paramStr);
    break;
  case  LOG:                         // Begin logging
    Serial.println("Logging...");
    mode = LOGGING;
    break;
  case  MENU: // Refresh menu        // Redraw our menu again
    showMainMenu();
    break;
  case  RECORD:                     // Set next item line number
    setLogRecordStart(paramInt);
    break;
  default: 
    Serial.println("Unknown command.");    // Bad command char
    noError = false;
  }  //End switch
  if (noError) getCurrentSettings("no");        // Do any updates but don't show us
  return noError;
}  // End doCommand



// End file Interface


































