/*
File Name: SD
 Contents: Code for managing SD card writes/reads.  Based on LadyAda logger code.
 Last modified: 1/5/11
 Programmer: John Vaughn
 Comments:  Some SD cards can be cranky about mounting and initializing properly.  Setting init(SPI_HALF_SPEED) seems to help as well as
 trying different cards.  Some high capacity SD Micro cards which cannot be read in older card readers still work in the logger. Go figure.
 */

#define SYNC_INTERVAL 1000 // mills between calls to sync()
#define LOG_INTERVAL 200
uint32_t syncTime = 0; // time of last sync()

// The global objects to talk to the SD card
Sd2Card card;
SdVolume volume;
SdFile root;
SdFile file;

// Initialize the SD card and open the root directory.  We pass in a filename which we try to open after initializing the card. Failures are reported 
// in return values.

// Try to initialize the SD card, and open the specified file.  Notice our filename parameter may be modified by this call.
// We try to generate filenames as logger00.txt, logger01.txt, etc until we can locate/open such a file.
int initSD(char fileName[])
{
  // Initialize the SD card
  if (!card.init(SPI_HALF_SPEED) || !volume.init(card) || !root.openRoot(volume) ) {
    Serial.println("No SD card."); 
    return CARD_ERROR;
  }  // Notify the user if no card is available

  // Create a new file based on our EEPROM
  // Use our supplied file name as a template for new files names if ours already exists.
  for (uint8_t i = 0; i < 100; i++) {
    fileName[6] = i/10 + '0';
    fileName[7] = i%10 + '0';
    if (file.open(root, fileName, O_CREAT | O_EXCL | O_WRITE)) break;
  }  // End of trying new file names

  if (!file.isOpen()) return FILE_ERROR;

  // Make sure we can sync with our file
  file.writeError = 0;
  checkSync();
  if (file.writeError || !file.sync()) {
    return WRITE_ERROR;
  }  // End if
  return NO_ERROR;
}  // End initSD

// Write a string to our file
int writeSDStr(char str[])
{
  file.writeError = NO_ERROR;    // clear print error
  file.print(str);
  return (checkSync() + file.writeError == NO_ERROR ? NO_ERROR : WRITE_ERROR);
}  // End writeSDStr

// Write an int to our file
int writeSDInt(int n) {
  file.writeError = NO_ERROR;    // clear print error
  file.print(n);
  return (checkSync() + file.writeError == NO_ERROR ? NO_ERROR : WRITE_ERROR);
}  // End writeSDInt

int checkSync() {
  //don't sync too often - requires 2048 bytes of I/O to SD card
  if ((millis() - syncTime) < SYNC_INTERVAL) return NO_ERROR;
  syncTime = millis();
  if (!file.sync()) return SYNC_ERROR; 
  else return NO_ERROR;
}  // End check file sync

// End file SD

