/*
File Name: SEEPROM
 Contents: Code for reading and writing to I2C Serial EEPROM on board bathouse logger shield.
 Last modified: 12/21/10
 Programmer: Josh Davis, Tomas Carvalho Silvia, John Vaughn.
 Comments: The 24AA1025 serial EEPROM chip uses a 7-bit address for the chip itself and an 8th bit for read/write
 command.  Both these parts make up a control byte sent to the chip. Details available through the Microchip data sheets.
 Note: Control byte of 1010 011 [0|1] codes into 101 0011 = 0x53 as we ignore the LSB R/W command here for our 7-bit address.  
 Thus a control byte addresses only half of the 128K bytes of EEPROM.  To address the other 64K, use 101 0111 = 0x57 as our
 chip address for the upper memory bank.
 */
 
 //#define SEEPROM address from control byte  // 7-bit address of the Serial EEPROM: 1010 0011x => 101 0011 => 0x53
#define SEEPROM_CHIP_ADDR 0x53
// Low bank for addresses 0x0000 - 0xFFFF
#define SEEPROM_HIGH_BANK 0x57
// High bank for addresses above 0xFFFF
#define ADDR_MAX 65536

// Decide which bank we are going to address here.  This is necessary as we may move between banks at addresses 64K - 128K.
int selectBankAddr(unsigned long seepromAddress) {
  return (seepromAddress >= ADDR_MAX ? SEEPROM_HIGH_BANK : SEEPROM_CHIP_ADDR);  // Quick selection between addresses
}  // selectBankAddr

// Wrapper to write a string to SEEPROM at global seepromAddress doing the device and storage address modifications for high/low banks.
// We avoid wrapping SEERPROM to overwrite earlier values.  For bank select and addesses above SEEPROM_MAX at 128K, we simply do nothing.
void writeSEEPROM(char buffer[]){
  unsigned int addr;
  int deviceAddress;
  for (int i = 0; i < strlen(buffer); i++) {
    addr = seepromCurrent%ADDR_MAX;          // Adjust the storage address if needed
    deviceAddress = selectBankAddr(seepromCurrent);  // Adjust the device address as needed
    i2c_eeprom_write_byte(deviceAddress, addr, buffer[i]);
    seepromCurrent++;
    delay(10);    // SEEPROM is slooooooooow to respond, determined by trial and error
  }  // End for
}  // End of writeSEEPROM

// Wrapper to get a string of specified size from SEEPROM.  Again, handles bank transitions like writing above.
// Note, long string buffer size here runs the risk of crashing this program due to memory conflicts. Note: Parameter buffer
// must be at least dataSize + 1 in length.
void readSEEPROM(long seepromAddress, int dataSize, char buffer[]){
  unsigned int addr;
  int deviceAddress;
  int i = 0;
  while (i < dataSize) {
    addr = seepromAddress%ADDR_MAX;
    deviceAddress = selectBankAddr(seepromAddress);
    buffer[i++] = i2c_eeprom_read_byte(deviceAddress, addr);
    seepromAddress++;
    delay(10);  // SEEPROM is slooooooooow to respond, determined by trial and error
  }  // End for
  buffer[i] = NULL_CHAR;  // Add string terminator
}  // End of readSEEPROM

// Utility to dump contents of SEEPROM to serial display or SD card, primarily for debugging.  Assumes good SD card mounted.
boolean showSEEPROM(long startAddr, long bytes, int sendTo) {
  char buffer[STRING_MAX+1];                      // Buffer declared of enlarged length to handle reading STRING_MAX bytes.
  long addr = startAddr;
  while ( (addr + STRING_MAX) < (startAddr + bytes) ) {
    readSEEPROM(addr, STRING_MAX, buffer);
    if (sendTo == SERIAL) Serial.print(buffer);
    if (sendTo == SD) writeSDStr(buffer);
    addr += STRING_MAX;
  }  // End while
  readSEEPROM(addr, bytes%STRING_MAX, buffer);
   if (sendTo == SERIAL) Serial.println(buffer);
   if (sendTo == SD) writeSDStr(buffer);
   return true;
}  // End of showSEEPROM

// Write the single byte data given to the specified device at the specified address
void i2c_eeprom_write_byte( int deviceaddress, unsigned int eeaddress, byte data ) {
  Wire.beginTransmission(deviceaddress);
  Wire.send((int)(eeaddress >> 8)); // MSB
  Wire.send((int)(eeaddress & 0xFF)); // LSB
  Wire.send(data);
  Wire.endTransmission();
}  // End of write_byte

// Read a single byte from the specified device address and data address
byte i2c_eeprom_read_byte( int deviceaddress, unsigned int eeaddress ) {
  byte rdata = 0xFF;
  Wire.beginTransmission(deviceaddress);
  Wire.send((int)(eeaddress >> 8)); // MSB
  Wire.send((int)(eeaddress & 0xFF)); // LSB
  Wire.endTransmission();
  Wire.requestFrom(deviceaddress,1);
  if (Wire.available()) rdata = Wire.receive(); 
  else Serial.println("EEPROM not ready");
  return rdata;
}  // End of read_byte

// End SEEPROM file








